@@ -1,5 +1,6 @@
+#line 1 "inc/Module/Install/AutoInstall.pm - /usr/local/lib/perl5/site_perl/5.8.2/Module/Install/AutoInstall.pm"
 # $File: //depot/cpan/Module-Install/lib/Module/Install/AutoInstall.pm $ $Author: autrijus $
-# $Revision: #9 $ $Change: 1341 $ $DateTime: 2003/03/09 23:28:03 $ vim: expandtab shiftwidth=4
+# $Revision: #12 $ $Change: 1481 $ $DateTime: 2003/05/07 10:41:22 $ vim: expandtab shiftwidth=4
 
 package Module::Install::AutoInstall;
 use Module::Install::Base; @ISA = qw(Module::Install::Base);
@@ -20,31 +21,35 @@ sub auto_install {
     my $self = shift;
     return if $self->{done}++;
 
-# ExtUtils::AutoInstall Bootstrap Code, version 6.
-AUTO:{my$p='ExtUtils::AutoInstall';my$v=0.45;$p->VERSION||0>=$v
+# ExtUtils::AutoInstall Bootstrap Code, version 7.
+AUTO:{my$p='ExtUtils::AutoInstall';my$v=0.49;$p->VERSION||0>=$v
 or+eval"use $p $v;1"or+do{my$e=$ENV{PERL_EXTUTILS_AUTOINSTALL};
 (!defined($e)||$e!~m/--(?:default|skip|testonly)/and-t STDIN or
 eval"use ExtUtils::MakeMaker;WriteMakefile(PREREQ_PM=>{'$p',$v}
 );1"and exit)and print"==> $p $v required. Install it from CP".
 "AN? [Y/n] "and<STDIN>!~/^n/i and print"*** Installing $p\n"and
-do{eval{require CPANPLUS;CPANPLUS::install $p};eval("use $p $v;
-1")||eval{require CPAN;CPAN::install$p};eval"use $p $v;1"or die
-"*** Please manually install $p $v from cpan.org first...\n"}}}
+do{if (eval '$>' and lc(`sudo -V`) =~ /version/){system('sudo',
+$^X,"-MCPANPLUS","-e","CPANPLUS::install $p");eval"use $p $v;1"
+||system('sudo', $^X, "-MCPAN", "-e", "CPAN::install $p")}eval{
+require CPANPLUS;CPANPLUS::install$p};eval"use $p $v;1"or eval{
+require CPAN;CPAN::install$p};eval"use $p $v;1"||die"*** Please
+manually install $p $v from cpan.org first...\n"}}}
 
     # Flatten array of arrays into a single array
-    my @core = map @$_, map @$_, grep ref, map $self->$_,
-               qw(build_requires requires);
+    my @core = map @$_, map @$_, grep ref,
+               $self->build_requires, $self->requires;
 
     ExtUtils::AutoInstall->import(
         (@core ? (-core => \@core) : ()), @_, $self->features
     );
 
+    $self->makemaker_args( ExtUtils::AutoInstall::_make_args() );
+
     my $class = ref($self);
     $self->postamble(
         "# --- $class section:\n" .
         ExtUtils::AutoInstall::postamble()
     );
-    $self->makemaker_args( ExtUtils::AutoInstall::_make_args() );
 }
 
 sub auto_install_now {
